#include <streamer/streamer.h>
#include <fab.h>
#include "common.h"

StreamElementType *_fab;

StreamElement *Stream_fab_create() {
  Stream_fab *element;

  element = (Stream_fab *)malloc(sizeof(Stream_fab));

  element->index = 0;
  element->pool = StreamBufferPool_create(2,0,0,8);

  element->element.flags = 0;
  element->element.inpadfunc = NULL; /* leave it up to general case */
  element->element.outpadfunc = NULL; /* ditto */
  element->element.chain = NULL; /* none */
  element->element.push = Stream_fab_push;
  element->element.inpads = NULL;
  element->element.outpads = NULL;

  element->out.type = 0;
  element->out.flags = 0;
  element->out.owner = (StreamElement *)element;
  element->out.peer = NULL;
  element->out.chain = NULL;
  /* register it */
  element->element.outpads =
    g_slist_append(element->element.outpads,&element->out);

  return (StreamElement *)element;
}

void Stream_fab_push(StreamElement *element) {
  Stream_fab *fab_element = (Stream_fab *)element;
  StreamBuffer *buf;

  if (fab_element->out.chain) {
    buf = StreamBuffer_create(fab_element->pool);
    sprintf(buf->data,"%d\n",fab_element->index++);
    DEBUG("fab created buffer %p\n",buf);

    DEBUG("fab about to chain to %p()\n",fab_element->out.chain);
    StreamPad_chain(&fab_element->out,buf);
  }
}


void init_StreamPlugin(StreamPlugin *plugin) {
  /* we have to do this because we're going to return static data */
  g_module_make_resident(plugin->module);

  /* fill in the StreamPlugin data */
  plugin->name = "fab";
  plugin->longname = "Fabrication filters";
  plugin->version = "0.0.0";
  plugin->elementtypes = NULL;
  plugin->enumtypes = NULL; /* simple case, we rely on default */

  /* create the element types */
  _fab = (StreamElementType *)g_malloc(sizeof(StreamElementType));
  _fab->plugin = plugin;
  _fab->name = "fab";
  _fab->longname = "Fabrication";
  _fab->version = "0.0.0";
  _fab->create = Stream_fab_create;

  /* add it to necessary lists */
  plugin->elementtypes = g_slist_append(plugin->elementtypes,_fab);
  StreamElementType_register(_fab);

  /* announce that we're here... */
  DEBUG("initialized plugin '%s'\n",StreamPlugin_getname(plugin));
}

