#include "streamer-private.h"

GSList *_StreamElementType_list;

void _StreamElement_init() {
  DEBUG("in _StreamElement_init()\n");
}

void StreamElementType_register(StreamElementType *type) {
  DEBUG("StreamElementType_register: registering %s\n",type->name);
  _StreamElementType_list = g_slist_append(_StreamElementType_list,type);
}

StreamPad *StreamElement_inpad(StreamElement *element,int idx) {
  /* insanity check */
  if (!element) {
    g_warning("StreamElement_inpad: NULL element!\n");
    return(NULL);
  }

  DEBUG("StreamElement_inpad: looking up inpad %d of %p\n",idx,element);
  if (element->inpadfunc) return((element->inpadfunc)(element,idx));
  return((StreamPad *)g_slist_nth_data(element->inpads,idx));
}

StreamPad *StreamElement_outpad(StreamElement *element,int idx) {
  StreamPad *pad;

  /* insanity check */
  if (!element) {
    g_warning("StreamElement_inpad: NULL element!\n");
    return(NULL);
  }

  DEBUG("StreamElement_outpad: looking up outpad %d of %p\n",idx,element);
  if (element->outpadfunc) return((element->outpadfunc)(element,idx));
  if (!element->outpads) g_warning("StreamElement_outpad: null padlist\n");
  pad = (StreamPad *)g_slist_nth_data(element->outpads,idx);
  if (!pad) g_warning("StreamElement_outpad: null pad\n");
  return(pad);
}

void StreamElement_connect(StreamElement *out,int idxout,
                           StreamElement *in,int idxin) {
  DEBUG("chaining %p:%i to %p:%i, with %p()\n",out,idxout,in,
         idxin,in->chain);
  StreamElement_outpad(out,idxout)->peer = StreamElement_inpad(in,idxin);
  StreamElement_inpad(in,idxin)->peer = StreamElement_outpad(out,idxout);

  StreamElement_outpad(out,idxout)->chain =
    StreamElement_inpad(in,idxin)->chain;
}

void StreamElement_push(StreamElement *element) {
  (element->push)(element);
}
