/* Copyright (C) 1998 Oregon Graduate Institute of Science & Technology
 * Portions copyright (c) 1995 Brown University.
 * Copyright (c) 1995 The Regents of the University of California.
 * 
 * See the file COPYRIGHT, which should have been supplied with this
 * package, for licensing details.  We may be contacted through email
 * at <quasar-help@cse.ogi.edu>.
 */

/*
 * global.c --
 *
 *      Global variables and related commands.
 *
 */

#include "global.h"
#include "video.h"
#include "dither.h"


/* Flag for gamma correction */
int gammaCorrectFlag = 0;
double gammaCorrect = 1.0;

/* Flag for chroma correction */
int chromaCorrectFlag = 0;
double chromaCorrect = 1.0;

/* Flag for high quality at the expense of speed */
#ifdef QUALITY
int qualityFlag = 1;
#else
int qualityFlag = 0;
#endif



/*
 *--------------------------------------------------------------
 *
 * InitEverything --
 *
 *	Initializes global tables and structures.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *      None.
 *
 *--------------------------------------------------------------
 */

void
InitEverything()
{
  DBUG_ENTER();
  InitCrop();

  LUM_RANGE = 8;
  CR_RANGE = CB_RANGE = 4;

  lum_values = (int *) malloc(LUM_RANGE*sizeof(int));
  cr_values = (int *) malloc(CR_RANGE*sizeof(int));
  cb_values = (int *) malloc(CB_RANGE*sizeof(int));
  DBUG_VOID_RETURN();
}


/*
 *--------------------------------------------------------------
 *
 * DestructEverything --
 *
 *	Destroys global tables and structures.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *      None.
 *
 *--------------------------------------------------------------
 */

void
DestructEverything()
{
  DBUG_ENTER();
  free(lum_values);
  free(cr_values);
  free(cb_values);
  DBUG_VOID_RETURN();
}
