/* Copyright (C) 1998 Oregon Graduate Institute of Science & Technology
 * Portions copyright (c) 1995 Brown University.
 * Copyright (c) 1995 The Regents of the University of California.
 * 
 * See the file COPYRIGHT, which should have been supplied with this
 * package, for licensing details.  We may be contacted through email
 * at <quasar-help@cse.ogi.edu>.
 */

#ifndef READFILE_H
#define READFILE_H

#include "global.h"

#ifdef HAVE_LIBSWQ
#include <swqueue.h>
#include <smba.h>
#endif

#ifdef HAVE_CHANNELS
#include <pipeline/channel.h>
#endif

/* Data Stream structure */

typedef struct DataStream {
  FILE *input;                                 /* stream comes from here     */
#ifdef HAVE_LIBSWQ
  swq *queue;
  smba *buf_area;
#endif
#ifdef HAVE_CHANNELS
  channel_src *channelin;
  channel_dst *channelout;
  int channel_EOF;
#endif // HAVE_CHANNELS
  int EOF_flag;                                /* stream is EOF              */
  int buf_length;                              /* Length of remaining buffer.*/
  int max_buf_length;                          /* Max length of buffer.      */
  unsigned int *buffer;                        /* Pointer to next byte in
						  buffer.                    */
  unsigned int *buf_start;                     /* Pointer to buffer start.   */
  int bit_offset;                              /* Bit offset in stream.      */
  unsigned int curBits;                        /* current bits               */
  int EOF_adjust;			       /* Byte adjustment for EOF.   */

#ifdef SYS_STREAM
  int sys_layer;                               /* -1 uninitialized,
	                                           0 video layer,
						   1 syslayer                */
  int num_left;
  int leftover_bytes;
#endif
} DataStream;


/* External declarations for bitstream i/o operations. */
extern unsigned int bitMask[];
extern unsigned int nBitMask[];
extern unsigned int rBitMask[];
extern unsigned int bitTest[];

#ifdef NO_SANITY_CHECKS
#define get_bits1(result)                                                 \
{                                                                         \
  result = ((data_stream->curBits & 0x80000000) != 0);                    \
  data_stream->curBits <<= 1;                                             \
  data_stream->bit_offset++;                                              \
                                                                          \
  if (data_stream->bit_offset & 0x20) {                                   \
    data_stream->bit_offset = 0;                                          \
    data_stream->buffer++;                                                \
    data_stream->curBits = *data_stream->buffer;                          \
    data_stream->buf_length--;                                            \
  }                                                                       \
}

#define get_bits2(result)                                                 \
{                                                                         \
  data_stream->bit_offset += 2;                                           \
                                                                          \
  if (data_stream->bit_offset & 0x20) {                                   \
    data_stream->bit_offset -= 32;                                        \
    data_stream->buffer++;                                                \
    data_stream->buf_length--;                                            \
    if (data_stream->bit_offset) {                                        \
      data_stream->curBits |=                                             \
	 (*data_stream->buffer >> (2 - data_stream->bit_offset));         \
    }                                                                     \
    result = ((data_stream->curBits & 0xc0000000) >> 30);                 \
    data_stream->curBits = *data_stream->buffer << data_stream->bit_offset;  \
  }                                                                       \
                                                                          \
  result = ((data_stream->curBits & 0xc0000000) >> 30);                   \
  data_stream->curBits <<= 2;                                             \
}

#define get_bitsX(num, mask, shift,  result)                              \
{                                                                         \
  data_stream->bit_offset += num;                                         \
                                                                          \
  if (data_stream->bit_offset & 0x20) {                                   \
    data_stream->bit_offset -= 32;                                        \
    data_stream->buffer++;                                                \
    data_stream->buf_length--;                                            \
    if (data_stream->bit_offset) {                                        \
      data_stream->curBits |= (*data_stream->buffer >>                    \
      (num - data_stream->bit_offset));                                   \
    }                                                                     \
    result = ((data_stream->curBits & mask) >> shift);                    \
    data_stream->curBits = *data_stream->buffer << data_stream->bit_offset;  \
  }                                                                       \
  else {                                                                  \
    result = ((data_stream->curBits & mask) >> shift);                    \
    data_stream->curBits <<= num;                                         \
  }                                                                       \
}
#else

#define get_bits1(result)                                                 \
{                                                                         \
  /* Check for underflow. */                                              \
                                                                          \
  if (data_stream->buf_length < 2) {                                      \
    correct_underflow(data_stream);                                       \
  }                                                                       \
  result = ((data_stream->curBits & 0x80000000) != 0);                    \
  data_stream->curBits <<= 1;                                             \
  data_stream->bit_offset++;                                              \
                                                                          \
  if (data_stream->bit_offset & 0x20) {                                   \
    data_stream->bit_offset = 0;                                          \
    data_stream->buffer++;                                                \
    data_stream->curBits = *data_stream->buffer;                          \
    data_stream->buf_length--;                                            \
  }                                                                       \
}

#define get_bits2(result)                                                 \
{                                                                         \
  /* Check for underflow. */                                              \
                                                                          \
  if (data_stream->buf_length < 2) {                                      \
    correct_underflow(data_stream);                                       \
  }                                                                       \
  data_stream->bit_offset += 2;                                           \
                                                                          \
  if (data_stream->bit_offset & 0x20) {                                   \
    data_stream->bit_offset -= 32;                                        \
    data_stream->buffer++;                                                \
    data_stream->buf_length--;                                            \
    if (data_stream->bit_offset) {                                        \
      data_stream->curBits |= (*data_stream->buffer >>                    \
      (2 - data_stream->bit_offset));                                     \
    }                                                                     \
    result = ((data_stream->curBits & 0xc0000000) >> 30);                 \
    data_stream->curBits = *data_stream->buffer << data_stream->bit_offset;  \
  }                                                                       \
                                                                          \
  result = ((data_stream->curBits & 0xc0000000) >> 30);                   \
  data_stream->curBits <<= 2;                                             \
}

#define get_bitsX(num, mask, shift,  result)                              \
{                                                                         \
  /* Check for underflow. */                                              \
                                                                          \
  if (data_stream->buf_length < 2) {                                      \
    correct_underflow(data_stream);                                       \
  }                                                                       \
  data_stream->bit_offset += num;                                         \
                                                                          \
  if (data_stream->bit_offset & 0x20) {                                   \
    data_stream->bit_offset -= 32;                                        \
    data_stream->buffer++;                                                \
    data_stream->buf_length--;                                            \
    if (data_stream->bit_offset) {                                        \
      data_stream->curBits |= (*data_stream->buffer >>                    \
      (num - data_stream->bit_offset));                                   \
    }                                                                     \
    result = ((data_stream->curBits & mask) >> shift);                    \
    data_stream->curBits = *data_stream->buffer << data_stream->bit_offset;  \
  }                                                                       \
  else {                                                                  \
   result = ((data_stream->curBits & mask) >> shift);                     \
   data_stream->curBits <<= num;                                          \
  }                                                                       \
}
#endif

#define get_bits3(result) get_bitsX(3,   0xe0000000, 29, result)
#define get_bits4(result) get_bitsX(4,   0xf0000000, 28, result)
#define get_bits5(result) get_bitsX(5,   0xf8000000, 27, result)
#define get_bits6(result) get_bitsX(6,   0xfc000000, 26, result)
#define get_bits7(result) get_bitsX(7,   0xfe000000, 25, result)
#define get_bits8(result) get_bitsX(8,   0xff000000, 24, result)
#define get_bits9(result) get_bitsX(9,   0xff800000, 23, result)
#define get_bits10(result) get_bitsX(10, 0xffc00000, 22, result)
#define get_bits11(result) get_bitsX(11, 0xffe00000, 21, result)
#define get_bits12(result) get_bitsX(12, 0xfff00000, 20, result)
#define get_bits14(result) get_bitsX(14, 0xfffc0000, 18, result)
#define get_bits16(result) get_bitsX(16, 0xffff0000, 16, result)
#define get_bits18(result) get_bitsX(18, 0xffffc000, 14, result)
#define get_bits32(result) get_bitsX(32, 0xffffffff,  0, result)

#define get_bitsn(num, result) get_bitsX((num), nBitMask[num], (32-(num)), result)

#ifdef NO_SANITY_CHECKS
#define show_bits32(result)                              		\
{                                                                       \
  if (data_stream->bit_offset) {					\
    result = data_stream->curBits | (*(data_stream->buffer+1) >>        \
	 (32 - data_stream->bit_offset));                               \
  }                                                                     \
  else {                                                                \
    result = data_stream->curBits;					\
  }                                                                     \
}

#define show_bitsX(num, mask, shift,  result)                           \
{                                                                       \
  int bO;                                                               \
  bO = data_stream->bit_offset + num;                                   \
  if (bO > 32) {                                                        \
    bO -= 32;                                                           \
    result = ((data_stream->curBits & mask) >> shift) |                 \
                (*(data_stream->buffer+1) >> (shift + (num - bO)));     \
  }                                                                     \
  else {                                                                \
    result = ((data_stream->curBits & mask) >> shift);                  \
  }                                                                     \
}

#else
#define show_bits32(result)                               		\
{                                                                       \
  /* Check for underflow. */                                            \
  if (data_stream->buf_length < 2) {                                    \
    correct_underflow(data_stream);                                     \
  }                                                                     \
  if (data_stream->bit_offset) {						\
    result = data_stream->curBits | (*(data_stream->buffer+1) >>        \
    (32 - data_stream->bit_offset));		                        \
  }                                                                     \
  else {                                                                \
    result = data_stream->curBits;					\
  }                                                                     \
}

#define show_bitsX(num, mask, shift, result)                            \
{                                                                       \
  int bO;                                                               \
                                                                        \
  /* Check for underflow. */                                            \
  if (data_stream->buf_length < 2) {                                    \
    correct_underflow(data_stream);                                     \
  }                                                                     \
  bO = data_stream->bit_offset + num;                                   \
  if (bO > 32) {                                                        \
    bO -= 32;                                                           \
    result = ((data_stream->curBits & mask) >> shift) |                 \
                (*(data_stream->buffer+1) >> (shift + (num - bO)));     \
  }                                                                     \
  else {                                                                \
    result = ((data_stream->curBits & mask) >> shift);                  \
  }                                                                     \
}
#endif

#define show_bits1(result)  show_bitsX(1,  0x80000000, 31, result)
#define show_bits2(result)  show_bitsX(2,  0xc0000000, 30, result)
#define show_bits3(result)  show_bitsX(3,  0xe0000000, 29, result)
#define show_bits4(result)  show_bitsX(4,  0xf0000000, 28, result)
#define show_bits5(result)  show_bitsX(5,  0xf8000000, 27, result)
#define show_bits6(result)  show_bitsX(6,  0xfc000000, 26, result)
#define show_bits7(result)  show_bitsX(7,  0xfe000000, 25, result)
#define show_bits8(result)  show_bitsX(8,  0xff000000, 24, result)
#define show_bits9(result)  show_bitsX(9,  0xff800000, 23, result)
#define show_bits10(result) show_bitsX(10, 0xffc00000, 22, result)
#define show_bits11(result) show_bitsX(11, 0xffe00000, 21, result)
#define show_bits12(result) show_bitsX(12, 0xfff00000, 20, result)
#define show_bits13(result) show_bitsX(13, 0xfff80000, 19, result)
#define show_bits14(result) show_bitsX(14, 0xfffc0000, 18, result)
#define show_bits15(result) show_bitsX(15, 0xfffe0000, 17, result)
#define show_bits16(result) show_bitsX(16, 0xffff0000, 16, result)
#define show_bits17(result) show_bitsX(17, 0xffff8000, 15, result)
#define show_bits18(result) show_bitsX(18, 0xffffc000, 14, result)
#define show_bits19(result) show_bitsX(19, 0xffffe000, 13, result)
#define show_bits20(result) show_bitsX(20, 0xfffff000, 12, result)
#define show_bits21(result) show_bitsX(21, 0xfffff800, 11, result)
#define show_bits22(result) show_bitsX(22, 0xfffffc00, 10, result)
#define show_bits23(result) show_bitsX(23, 0xfffffe00,  9, result)
#define show_bits24(result) show_bitsX(24, 0xffffff00,  8, result)
#define show_bits25(result) show_bitsX(25, 0xffffff80,  7, result)
#define show_bits26(result) show_bitsX(26, 0xffffffc0,  6, result)
#define show_bits27(result) show_bitsX(27, 0xffffffe0,  5, result)
#define show_bits28(result) show_bitsX(28, 0xfffffff0,  4, result)
#define show_bits29(result) show_bitsX(29, 0xfffffff8,  3, result)
#define show_bits30(result) show_bitsX(30, 0xfffffffc,  2, result)
#define show_bits31(result) show_bitsX(31, 0xfffffffe,  1, result)

#define show_bitsn(num,result) show_bitsX((num), (0xffffffff << (32-(num))), (32-(num)), result)

#ifdef NO_SANITY_CHECKS
#define flush_bits32                                                  \
{                                                                     \
  data_stream->buffer++;                                              \
  data_stream->buf_length--;                                          \
  data_stream->curBits = *data_stream->buffer  << data_stream->bit_offset;  \
}


#define flush_bits(num)                                               \
{                                                                     \
  data_stream->bit_offset += num;                                     \
                                                                      \
  if (data_stream->bit_offset & 0x20) {                               \
    data_stream->bit_offset -= 32;                                    \
    data_stream->buffer++;                                            \
    data_stream->buf_length--;                                        \
    data_stream->curBits = *data_stream->buffer  << data_stream->bit_offset;\
  }                                                                   \
  else {                                                              \
    data_stream->curBits <<= num;                                     \
  }                                                                   \
}
#else
#define flush_bits32                                                  \
{                                                                     \
  if (data_stream  == NULL) {                                         \
    /* Deal with no vid stream here. */                               \
  }                                                                   \
                                                                      \
  if (data_stream->buf_length < 2) {                                  \
    correct_underflow(data_stream);                                   \
  }                                                                   \
                                                                      \
  data_stream->buffer++;                                              \
  data_stream->buf_length--;                                          \
  data_stream->curBits = *data_stream->buffer  << data_stream->bit_offset;  \
}

#define flush_bits(num)                                               \
{                                                                     \
  if (data_stream== NULL) {                                           \
    /* Deal with no vid stream here. */                               \
  }                                                                   \
                                                                      \
  if (data_stream->buf_length < 2) {                                  \
    correct_underflow(data_stream);                                   \
  }                                                                   \
                                                                      \
  data_stream->bit_offset += num;                                     \
                                                                      \
  if (data_stream->bit_offset & 0x20) {                               \
    data_stream->buf_length--;                                        \
    data_stream->bit_offset -= 32;                                    \
    data_stream->buffer++;                                            \
    data_stream->curBits = *data_stream->buffer << data_stream->bit_offset;  \
  }                                                                   \
  else {                                                              \
    data_stream->curBits <<= num;                                     \
  }                                                                   \
}
#endif

DataStream *NewDataStream(unsigned int buffer_len);
void DestroyDataStream(DataStream *data);
void SeekStream(DataStream *data_stream, long seekValue);
long TellStream(DataStream *data_stream);
void correct_underflow(DataStream *data_stream);
int next_bits(int num, unsigned int mask, DataStream *data_stream);
int back_bits(int num, DataStream *data_stream);
int get_more_data(DataStream *data_stream);
int pure_get_more_data(unsigned int **bs_ptr, int *max_length_ptr, int *length_ptr, unsigned int **buf_ptr, DataStream *data_stream);

#ifdef SYS_STREAM
void clear_data_stream(DataStream *data_stream);
int read_sys(DataStream *data_stream, unsigned int start);
int ReadStartCode(unsigned int *startCode, DataStream *data_stream);
int ReadPackHeader(double *systemClockTime, unsigned long *muxRate, DataStream *data_stream);
int ReadSystemHeader(DataStream *data_stream);

int find_start_code(FILE *input);

int ReadPacket(unsigned char packetID, DataStream *data_stream);
void ReadTimeStamp(unsigned char *inputBuffer, unsigned char *hiBit, unsigned long *low4Bytes);
void ReadSTD(unsigned char *inputBuffer, unsigned char *stdBufferScale, unsigned long *stdBufferSize);
void ReadRate(unsigned char *inputBuffer, unsigned long *rate);

int MakeFloatClockTime(unsigned char hiBit, unsigned long low4Bytes, double *floatClockTime);

#endif /* SYS_STREAM */

#endif /* READFILE_H */
